#coding=utf8

# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.

'''
CloudControl alias related command
'''

from cloudcontrol.cli.exception import *
from cloudcontrol.cli.command import OptionCommand

import re

class Command_alias(OptionCommand):
    '''Show or create alias'''

    def __init__(self, cli, argv0):
        OptionCommand.__init__(self, cli, argv0)
        self.set_usage("%prog [options] [name] [value]")

    def __call__(self, argv):
        # args parse
        self.parse_args(argv)
        _value = None
        if len(self.args) == 0:
            _alias = sorted(self.cli.aliases.keys())
        elif len(self.args) == 1:
            _alias = [ self.args[0] ]
        elif len(self.args) == 2:
            _alias = self.args[0]
            _value = self.args[1]
        else:
            raise cmdBadArgument()
        # printing
        if _value is None:
            for a in _alias:
                if a in self.cli.aliases:
                    self.printer.out("%s \"%s\""%(a, re.sub("\"", "\\\"", self.cli.aliases[a])))
                else:
                    self.printer.warn("No alias %s"%a)
        # editing
        else:
            self.cli.aliases[_alias] = _value
            try:
                self.cli.aliases.save(self.cli.settings.get("alias", None))
            except Exception as e:
                raise cmdError(e)


class Command_unalias(OptionCommand):
    '''Remove an alias'''

    def __init__(self, cli, argv0):
        OptionCommand.__init__(self, cli, argv0)
        self.set_usage("%prog [options] [name]")

    def __call__(self, argv):
        # parse args
        self.parse_args(argv)
        if len(self.args) != 1:
            raise cmdBadArgument()
        # check alias existance
        if self.args[0] not in self.cli.aliases:
            raise cmdBadArgument("%s: No such alias"%self.args[0])
        # deleting aliases
        del self.cli.aliases[self.args[0]]
        try:
            self.cli.aliases.save(self.cli.settings.get("alias", None))
        except Exception as e:
            raise cmdError(e)
