#coding=utf8

# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.

'''
CloudControl right releated commands
'''

import tempfile
import subprocess
import os
import re
import pprint

from cloudcontrol.cli.exception import *
from sjrpc.core.exceptions import *
from cloudcontrol.cli.printer import Printer, color
from cloudcontrol.cli.command import Command, RemoteCommand

class Command_rights(RemoteCommand):
    '''List account rights'''

    def __init__(self, cli, argv0):
        RemoteCommand.__init__(self, cli, argv0)
        self.set_usage("%prog [options]")

    def __call__(self, argv):
        # ask server
        al = self.rpc.call("loadrights")
        # display answer
        for r in al:
            self.printer.out("%(match)s\t%(method)s\t%(tql)s\t%(action)s" % r)

    def remote_functions(self):
        return set(('loadrights',))


class Command_editrights(RemoteCommand):
    ''' Edit rights '''

    def __init__(self, cli, argv0):
        RemoteCommand.__init__(self, cli, argv0)
        self.set_usage("%prog [options]")

    def indent(self, rights, space):
        indents = {}
        fields = ('match', 'method', 'tql', 'action')
        for f in fields:
            cur = 0
            for r in rights:
                cur = max(cur, len(r[f]))
            indents[f] = cur + space
        return indents

    def edit(self, out):
        editor = os.environ.get('EDITOR', '/bin/nano')
        subprocess.call([editor, out.name])
        out.seek(0)
        new_rights = []
        nb = 1
        for line in out:
            line = line.lstrip().rstrip()
            if len(line) == 0 or line.startswith('#'):
                continue
            items = line.split()
            if len(items) < 4 or (len(items) > 4 and not items[5].startswith('#')):
                while True:
                    s = raw_input("Error: bad file format at line %d, do you want to reedit ? [yes/no] " % nb)
                    answer = s.lstrip().rstrip()
                    if answer == 'yes':
                        return self.edit(out)
                    elif answer == 'no':
                        raise cmdError('Bad file format at line %d' % nb)
                    else:
                        self.printer.out("Please answer by yes or no")
            new_rights.append({'match': items[0], 'method': items[1], 'tql': items[2], 'action': items[3]})
            nb += 1
        return new_rights

    def __call__(self, argv):
        with tempfile.NamedTemporaryFile(dir='/tmp') as temp:
            al = self.rpc.call("loadrights")
            indents = self.indent(al, 2)
            temp.write("# match%smethod%stql%saction\n" % (' '*(indents['match'] - len('match') - 2),
                                                             ' '*(indents['method'] - len('method')),
                                                             ' '*(indents['tql'] - len('tql'))))
            for r in al:
                temp.write("%s%s%s%s%s%s%s\n" % (r['match'],
                                                   ' '*(indents['match'] - len(r['match'])),
                                                   r['method'],
                                                   ' '*(indents['method'] - len(r['method'])),
                                                   r['tql'],
                                                   ' '*(indents['tql'] - len(r['tql'])),
                                                   r['action']))
            temp.flush()
            new_rights = self.edit(temp)
            temp.close()
            self.rpc.call('saverights', new_rights)

    def remote_functions(self):
        return set(('saverights',))
