#coding=utf8

# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.

'''
CloudControl time command
'''

import time as systime
import datetime

from cloudcontrol.cli.exception import *
from sjrpc.core.exceptions import *
from cloudcontrol.cli.printer import Printer, color
from cloudcontrol.cli.command import Command


class Command_time(Command):
    '''Show execution time'''

    def __init__(self, cli, argv0):
        Command.__init__(self, cli, argv0)

    def __call__(self, argv):
        if len(argv) < 2:
            raise cmdBadArgument()
        # Substitute aliases
        self.args = self.cli.aliases.substitute(argv[1:])
        # Init time
        t0 = systime.time()
        try:
            # Execute command
            self.cli.commands(self.args)
        except KeyboardInterrupt:
            raise
        except cliException:
            raise
        finally:
            t1 = systime.time()
            # Compute elapsed time
            dt = int(t1 - t0)
            self.printer.out("Execution time: %s" % datetime.timedelta(seconds=dt))

    def usage(self):
        return "Usage: time <command>"
