#coding=utf8

# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.

'''
CloudControl watch command
'''

import sys
import time

from cloudcontrol.cli.exception import *
from sjrpc.core.exceptions import *
from cloudcontrol.cli.printer import Printer, color
from cloudcontrol.cli.command import OptionCommand
from cStringIO import StringIO

class Command_watch(OptionCommand):
    '''Show output of a repeated command'''

    def __init__(self, cli, argv0):
        OptionCommand.__init__(self, cli, argv0)
        self.set_usage("%prog [options] command [options]")
        self.add_option("-n", "--interval", dest="interval", type="float", default=1.0,
                        help="Define interval in seconds")

    def __call__(self, argv):
        # Parse argline
        self.parse_args(argv)
        if len(self.args) == 0:
            raise cmdBadArgument()
        if self.options.interval < 0.5:
            raise cmdBadArgument("Interval must be >= 0.5")
        while True:
            # Substitute aliases
            self.args = self.cli.aliases.substitute(self.args)
            # Put the fd of a buffer in sys.stdout and sys.err instead of the tty fd
            sys.stderr = sys.stdout = StringIO()
            try:
                # Clear tty
                self.printer.clear()
                # Execute command and write output in the buffer
                self.cli.commands(self.args)
                # Print the buffer
                self.printer.out(sys.stdout.getvalue(), fd=sys.__stdout__)
                # Wait
                time.sleep(self.options.interval)
            except KeyboardInterrupt:
                raise
            except cliException:
                raise
            finally:
                sys.stdout = sys.__stdout__
                sys.stderr = sys.__stderr__
